package cz.zweistein.df.soundsense.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;
import java.util.logging.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.sun.org.apache.xerces.internal.parsers.DOMParser;

import cz.zweistein.df.soundsense.util.LoggerSource;

public class SoundsXML {
	private static Logger logger = LoggerSource.logger;
	
	private List<Sound> sounds;

	public SoundsXML(String directory) {
		
		this.sounds = new LinkedList<Sound>();
		this.loadDirectory(directory);
		
	}
	
	private void loadDirectory(String directory) {
		logger.info("Scanning directory '"+directory+"'.");
		File dir = new File(directory);
		
		String[] files = dir.list();
		
		if (files == null) {
			logger.info("'"+directory+"' is empty or invalid? Ignoring.");
		} else {
			Arrays.sort(files);
			for (int i = 0; i < files.length; i++) {
				
				String fileName = files[i];
				
				if (new File(directory+fileName).isDirectory()) {
					this.loadDirectory(directory+fileName+"/");
				} else if (fileName.endsWith(".xml")) {
				
					try {
						logger.info("Loading config "+directory+fileName);
						this.loadFile(directory+fileName);
					} catch (Exception e) {
						logger.severe("Failed to load "+ fileName + ": " + e.toString());
					}
				
				} else {
					logger.finest("'"+fileName+"' is not configuration file.");
				}
			}
		}
		
	}
	
	private void loadFile(String fileName) throws SAXException, IOException {
		
		Document doc = null;
		InputSource source = new InputSource(new FileInputStream(fileName));
		DOMParser parser = new DOMParser();
		parser.parse(source);
		doc = parser.getDocument();
		
		NodeList soundTags = doc.getElementsByTagName("sound");
		
		for (int i = 0; i < soundTags.getLength(); i++) {
			Node soundNode = soundTags.item(i);
			
			String logPattern = null;
			Node logPatternNode = soundNode.getAttributes().getNamedItem("logPattern");
			if (logPatternNode != null) {
				logPattern = logPatternNode.getNodeValue();
			} else {
				logger.fine("Sound tag without 'logPattern' attribute encountered.");
			}
			
			boolean isMusic = false;
			if (soundNode.getAttributes().getNamedItem("music") != null) {
				String music = soundNode.getAttributes().getNamedItem("music").getNodeValue();
				if ("true".equals(music)) {
					isMusic = true;
				} else {
					logger.fine("Music parameter '"+music+"' not recogized.");
				}
			}
			
			MusicLoop musicLoop = null;
			if (soundNode.getAttributes().getNamedItem("loop") != null) {
				String loop = soundNode.getAttributes().getNamedItem("loop").getNodeValue();
				if ("true".equals(loop) || "start".equals(loop)) {
					musicLoop = MusicLoop.START_LOOPING;
				} else if ("stop".equals(loop) ) {
					musicLoop = MusicLoop.STOP_LOOPING;
				} else {
					logger.fine("Loop parameter '"+loop+"' not recogized.");
				}
			}
			
			boolean isSpeech = false;
			if (soundNode.getAttributes().getNamedItem("speech") != null) {
				String speech = soundNode.getAttributes().getNamedItem("speech").getNodeValue();
				if ("true".equals(speech)) {
					isSpeech = true;
				} else {
					logger.fine("Speech parameter '"+speech+"' not recogized.");
				}
			}
			
			Integer concurency = null;
			if (soundNode.getAttributes().getNamedItem("concurency") != null) {
				String concurencyText = soundNode.getAttributes().getNamedItem("concurency").getNodeValue();
				try {
					concurency = Integer.parseInt(concurencyText);
				} catch (NumberFormatException e) {
					logger.info("Concurency '"+concurencyText+"' for '"+logPattern+"' is not recognized as a number.");
				}
			}
			
			List<String> soundFiles = new ArrayList<String>();
			
			NodeList soundElements = soundNode.getChildNodes();
			for (int j = 0; j < soundElements.getLength(); j++) {
				Node configNode = soundElements.item(j);
				String name = configNode.getLocalName();
				
				if ("soundFile".equals(name)) {
					String soundFile = new File(fileName).getParent()+"/"+configNode.getAttributes().getNamedItem("fileName").getNodeValue();
					if (new File(soundFile).exists()) {
						soundFiles.add(soundFile);
					} else {
						logger.info("Did not find "+soundFile+", ignoring.");
					}
				}
			}
			
			if (soundFiles.size() == 0 && (isSpeech == false && musicLoop != MusicLoop.STOP_LOOPING)) {
				logger.info("Sound for '"+logPattern+"' does not contain any soundFiles and is not speech or end of loop, ignoring.");
			} else {
				this.sounds.add(new Sound(soundFiles, logPattern, musicLoop, isMusic, concurency, isSpeech));
				logger.finest("Added sound for "+logPattern);
			}
		}
		
	}

	public List<Sound> getSounds() {
		return this.sounds;
	}
	
}
